from collections import deque
from typing import Callable, List
from .components import components, Component

class ClickBuilder():
    def __init__(self, writer, m_graph, eid):
        self.writer = writer
        self.m_graph = m_graph
        self.g = self.m_graph.get_graph()
        self.internal_map = {}
        self.default_internal_map()
        self.emulation_id = eid

    def default_internal_map(self):
        moa_types = self.m_graph.get_moa_graph_types()
        for mtype in moa_types:
            self.internal_map[mtype] = []

    def build_elements(self):
        roots = self.m_graph.get_roots()

        node_queue = deque()
        node_queue.extend(roots)
        while node_queue:
            # nodes are strings which represent graph nodes
            # graph nodes are dictionaries of properties for those nodes
            node = node_queue.popleft()
            g_node = self.g.nodes[node]
            if g_node['visited']:
                continue

            g_node['visited'] = True
            node_queue.extend(self.g.neighbors(node))

            print("BQ: ", node)
            self.internal_map[type(g_node['info'])].append(node)

    def write_src(self, extract: Callable[[Component], List[str]], what: str):

        for typ, types in self.internal_map.items():
            elements = []
            for node in types:

                # get the moa object from the graph
                mo = self.g.nodes[node]['info']

                # get the component constructor for the moa object type and
                # instantiate an instance
                c = components[type(mo)](node, mo, self.g)

                # encode the element as click source code
                src = extract(c)

                if typ != 'link':
                    elements += src
                else:
                    self.writer.write_elements(
                        src, (("%s %s")%(node, what)).title())

            if elements == []:
                continue

            self.writer.write_elements(
                elements, ("%s %s"%(typ.__name__, what)).title())

        return True

    def write_elements(self):
        return self.write_src(
            lambda x: x.encode_elements(), "elements"
        )

    def write_pipelines(self):
        return self.write_src(
            lambda x: x.encode_pipelines(self.g), "pipelines"
        )
