import sys
import argparse
import random
import grpc
from moa.proto import moa_pb2
from moa.proto import moa_pb2_grpc

REQUEST_MAP = {"new_emulation": moa_pb2.NewEmulationRequest,
               "add_to_emulation": moa_pb2.AddToEmulationRequest,
               "update_emulation": moa_pb2.UpdateEmulationRequest,
               "start_emulation": moa_pb2.StartEmulationRequest,
               "stop_emulation": moa_pb2.StopEmulationRequest,
               "discard_emulation": moa_pb2.DiscardEmulationRequest,
               "shutdown_moa": moa_pb2.ShutdownRequest,
               "update_interfaces": moa_pb2.InterfaceUpdateRequest,
               "add_vxlans": moa_pb2.VXLANsRequest,
               "remove_vxlans": moa_pb2.VXLANsRequest,
               "finalize_emulation": moa_pb2.FinalizeEmulationRequest}



def run(command, rq):
    with grpc.insecure_channel('localhost:50052') as channel:
        #pylint: disable=unused-variable
        stub = moa_pb2_grpc.MoaStub(channel)
        code = "stub.%s(rq)" % command
        response = eval(code)

    if response.success:
        print("Command Success")
    else:
        print("Error %d - %s" % (response.error_number, response.error_string))

def generate_request(args):
    request = None
    if (args.command == "new_emulation" or
        args.command == "add_to_emulation" or
        args.command == "update_emulation"):
        json_blob = ""
        eid = 0
        if args.eid:
            eid = args.eid
        else:
            eid = random.randint(1, 10000)

        emulator = 'click'

        with open(args.json_file) as fh:
            json_blob = fh.read()

        print("Emulation ID is %d\n" % eid)
        if args.command == "new_emulation":
            request = REQUEST_MAP[args.command](emulation_id=eid,
                                                emulator=emulator,
                                                emulation_blob=json_blob)
        else:
            request = REQUEST_MAP[args.command](emulation_id=eid,
                                                emulation_blob=json_blob)
    elif args.command == "update_interfaces":
        json_blob = ""
        with open(args.json_file) as fh:
            json_blob = fh.read()

        request = REQUEST_MAP[args.command](interface_blob=json_blob)
    elif (args.command == "AddVXLANs" or args.command == "RemoveVXLANs"):
        json_blob = ""

        if args.interface == "":
            print("Interface needs to be set to Add/Remove VXLANS")
            sys.exit(-1)

        with open(args.json_file) as fh:
            json_blob = fh.read()

        request = REQUEST_MAP[args.command](vxlan_blob=json_blob, interface_name=args.interface)

    elif args.command == "shutdown_moa":
        request = REQUEST_MAP[args.command](cleanAll=True)
    else:
        if not args.eid:
            print("Error - Please specify an Emulation ID to Start/Stop/Discard\n")
        else:
            request = REQUEST_MAP[args.command](emulation_id=args.eid)

    return request

def list_commands():
    commands = """## Construct Emulation Commands:
NewEmulation      - Construct Link Emulation. Args are Emulation ID, JSON Blob 
                    of emulation objects, not needed to be complete.
AddToEmulation    - Add Emulation Objects to in construction emulation. Args are
                    emulation ID, JSON Bob of emulation objects.
FinalizeEmulation - Finish building an emulation. Args are Emulation ID

## Update Emulation Commands
UpdateEmulation - Update a running Emulation. Args are Emulation ID and JSON
                  Blob of update

## Global Emulation Commands:
StartEmulation   - Start an Emulation.  Args are Emulation ID to start
StopEmulation    - Stop an Emulation.  Args are Emulation ID to stop
DiscardEmulation - Stop and Discard all information related to an Emulation.
                   Args are Emulation ID to Discard
ShutdownMoa      - Shutdown Moa Server.  No Args

## Configuration Information Commands:
UpdateInterfaces - Add interface information to Moa.  Args are Interface 
                   information blob
AddVXLANs        - Add VXLANs to an interface. Args are VXLAN blob, name of
                   interface to be added too.
RemoveVXLANs     - Remove VXLANS from an interface. Args are VXLAN blob, name of
                   interface to be removed from."""

    print(commands)


def validate_command(command):
    return command in REQUEST_MAP

if __name__ == '__main__':
    random.seed()

    PARSER = argparse.ArgumentParser()
    PARSER.add_argument("command", help="Command to send to Moa", type=str, default="")
    PARSER.add_argument("--list_commands", help="List available commands",
                        action="store_true", default=False)
    PARSER.add_argument("--eid", help="Emulation ID to reference with command.\n\
    For generation commands, if no id given, one will be created at random\n\
    For start/stop/discard commands, providing no id is an error", type=int)
    PARSER.add_argument("--server", help="Server to connect to.  Default is localhost",
                        type=str, default="localhost")
    PARSER.add_argument("--json_file", help="File with JSON Blob for command", type=str)
    PARSER.add_argument("--interface", help="Interface to add/remove VXLANs to/from", type=str)
    ARGS = PARSER.parse_args()

    if ARGS.list_commands:
        list_commands()
        sys.exit(0)

    if not validate_command(ARGS.command):
        list_commands()
        print("\n\nError: Unknown Command %s" % ARGS.command)
        sys.exit(-1)

    REQUEST = generate_request(ARGS)
    if REQUEST is None:
        sys.exit(-1)

    run(ARGS.command, REQUEST)
